<?php
/*
 *
 */

namespace Sikofitt\HathorBundle\Command;

use Sikofitt\HathorBundle\Common\RegexTrait;
use Symfony\Component\Console\Style\SymfonyStyle;
use Symfony\Component\Finder\Finder;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Helper\ProgressBar;
use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand as BaseCommand;
//use Sikofitt\HathorBundle\Common\Music\ID3;
use Sikofitt\PhpId3;

class ScannerCommand extends BaseCommand
{
    use RegexTrait;

    protected function configure()
    {
        $this
            ->setName('hathor:scan')
            ->setDescription('Scans Music')
            ->addOption('directory', 'd', InputOption::VALUE_OPTIONAL, 'Directory to scan for music.',__DIR__)
            ->addArgument('extensions',InputArgument::IS_ARRAY, 'Extensions to scan for separated by a space',['mp2','mp3','mp4','wma','ogg']);

    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $container = $this->getContainer();
        $em = $container->get('doctrine');

        $extensions = $input->getArgument('extensions');

        // \b(\.mp2|\.mp3|\.mp4|\.wma|\.ogg)\b$
        $this->add('/\b(');
        $extensionRegex = '/\b(';
        foreach($extensions as $extension) {
            $this->add(sprintf('\.%s|',$extension));
            $extensionRegex .= sprintf('\.%s|',$extension);
        }

        $this->add(')\b$/i');
        $regex = $this->build();
        $extensionRegex[strlen($extensionRegex)-1] = ')';
        $extensionRegex .= '\b$/i';

        $output->writeln("\n Please wait ... finding files.\n");
        $finder = new Finder();
        $finder
            ->files()
            ->in($input->getOption('directory'))
            ->ignoreDotFiles(true)
            ->ignoreUnreadableDirs(true)
            ->name($extensionRegex);
        $progress = new ProgressBar($output, $finder->count());
        $cols = shell_exec('tput cols');
        $maxCount = strlen($finder->count()) * 2;
        $otherCount = 12;
        $progress->setBarWidth($cols - ($maxCount+$otherCount));
        $format = $this->getCustomFormat();
        $progress->setFormat(
            $format->message .
            $format->current .
            $format->currentMaxSep .
            $format->max .
            $format->bar .
            $format->percent .
            $format->elapsed .
            $format->elapsedEstimatedSep .
            $format->estimated .
            $format->memory
        );
        $progress->clear();
        $progress->start();
        $getID3 = new PhpId3;
        $progress->setBarCharacter($format->barCharacter);
        $progress->setProgressCharacter($format->progressCharacter);
        $progress->setEmptyBarCharacter($format->emptyBarCharacter);
        $io = new SymfonyStyle($input, $output);

        foreach ($finder as $file) {

            $fileLength = mb_strlen($file->getFilename());
            if($fileLength > ($cols - $progress->getBarWidth()) + $otherCount)
            {
                $fileName = substr($file->getFilename(), 0, (int)$cols - $otherCount);

            } else {
               $fileName =  $file->getFilename();
            }
            $currentFile = str_pad($fileName .  ' ...', ($cols - $otherCount + strlen('...') + 1 ), ' ');

            $progress->setMessage($currentFile);

            $getID3->setFileName($file->getRealPath());
            try {
                $allMusicData[] = $getID3->getInfo ();
            } catch(\Exception $e) {
                $io->warning(['','Exception was caught.', 'Message : ' . $e->getMessage()]);


                continue;
            }

            $progress->advance();

        }
        $progress->setMessage('Finished!');
        $progress->finish();
        $io->writeln(PHP_EOL);
        //$io->warning($regex);
        $io->success('Finished!');
        //var_dump($allMusicData);



    }

    private function getCustomFormat() {
        return (object)[
            'barCharacter' => '<fg=cyan;options=bold>#</>',
            'progressCharacter' => '<fg=cyan>#</>',
            'emptyBarCharacter' => '<fg=cyan>#</>',
            'message' => ' <fg=white;options=bold>%message%</>' . PHP_EOL,
            'current' => ' <fg=cyan>%current%</>',
            'currentMaxSep' => '<fg=black;options=bold>/</>',
            'max' => '<fg=cyan;options=bold>%max%</>',
            'percent' => ' <fg=cyan;options=bold>%percent:3s%</><fg=cyan>%</>' .PHP_EOL,
            'elapsed' => ' %elapsed:6s% Elapsed',
            'estimated' => '%estimated:-6s% Remaining ',
            'elapsedEstimatedSep' => ' / ',
            'memory' => '(%memory:6s%)',
            'bar' => ' <fg=cyan>[</>%bar%<fg=cyan>]</>',
        ];
    }
}
